@@ -13,6 +13,7 @@ from .exec_core import SimulatorValidator
 from .results.data_model import TestCaseResult, TestCaseResultType, TestResultsReport  # noqa: F401
 from .results.io import write_test_results
 from biosimulators_utils.biosimulations.utils import validate_biosimulations_api_response
+from biosimulators_utils.config import Colors
 from biosimulators_utils.gh_action.data_model import Comment, GitHubActionCaughtError  # noqa: F401
 from biosimulators_utils.gh_action.core import GitHubAction, GitHubActionErrorHandling
 from biosimulators_utils.image import get_docker_image
@@ -23,6 +24,8 @@ from natsort import natsort_keygen
 import biosimulators_utils.image
 import biosimulators_utils.simulator.io
 import requests
+import requests.exceptions
+import termcolor
 
 
 __all__ = [
@@ -46,6 +49,9 @@ def get_uncaught_exception_msg(exception):
         Comment(text=str(exception), error=True),
         Comment(text=('The complete log of your validation/submission job, including further information about the failure, '
                       + 'is available [here]({}).'.format(gh_action_run_url))),
+        Comment(text=('If you chose to validate your Docker image, the results of the validation of your image will be '
+                      'available shortly as a JSON file. A link to this file will be available from the "Artifacts" '
+                      'section at the bottom of [this page]({}).'.format(gh_action_run_url))),
         Comment(text='Once you have fixed the problem, edit the first block of this issue to re-initiate this validation.'),
         Comment(text=('The BioSimulators Team is happy to help. '
                       'Questions and feedback can be directed to the BioSimulators Team by posting comments to this issues that '
@@ -83,7 +89,7 @@ class ValidateCommitSimulatorGitHubAction(GitHubAction):
         self.reset_issue_labels(self.issue_number, [IssueLabel.validated.value, IssueLabel.invalid.value, IssueLabel.action_error.value])
 
         # get specifications of simulator and validate simulator
-        specifications, test_results = self.exec_core(submission)
+        specifications, test_results = self.exec_core(submission, submitter)
 
         # label issue as validated
         self.add_labels_to_issue(self.issue_number, [IssueLabel.validated.value])
@@ -140,8 +146,13 @@ class ValidateCommitSimulatorGitHubAction(GitHubAction):
         actions.append('validating the specifications of your simulator')
         if submission.validate_image:
             actions.append('validating your Docker image')
+            test_results_msg = (
+                ' The results of the validation of your tool will also be saved as a JSON file.'
+                'A link to this file will be available from the "Artifacts" section at the bottom of this page.'
+            )
         else:
             not_actions.append('You have chosen not to have the Docker image for your simulator validated.')
+            test_results_msg = ''
         if submission.commit_simulator:
             job_type = 'submission'
             actions.append('committing your simulator to the BioSimulators registry')
@@ -158,10 +169,10 @@ class ValidateCommitSimulatorGitHubAction(GitHubAction):
         return ('Thank you @{} for your submission to the BioSimulators simulator validation/submission system!\n\n'
                 'The BioSimulators validator bot is {}. {}\n\n'
                 'We will discuss any concerns with your submission in this issue.\n\n'
-                'A complete log of your simulator {} job is available [here]({}).\n\n'
-                ).format(submitter, actions, not_actions, job_type, self.get_gh_action_run_url())
+                'A complete log of your simulator {} job will be available [here]({}).{}\n\n'
+                ).format(submitter, actions, not_actions, job_type, self.get_gh_action_run_url(), test_results_msg)
 
-    def exec_core(self, submission):
+    def exec_core(self, submission, submitter):
         """ Validate simulator
 
         * Validate specifications
@@ -169,6 +180,7 @@ class ValidateCommitSimulatorGitHubAction(GitHubAction):
 
         Args:
             submission (:obj:`SimulatorSubmission`): simulator submission
+            submitter (:obj:`str`): GitHub id of the submitter
 
         Returns:
             :obj:`tuple`:
@@ -179,6 +191,11 @@ class ValidateCommitSimulatorGitHubAction(GitHubAction):
         # validate specifications
         specifications = biosimulators_utils.simulator.io.read_simulator_specs(
             submission.specifications_url, submission.specifications_patch)
+
+        # check permissions
+        self.validate_permissions(specifications['id'], specifications['name'], submitter)
+
+        # indicate that specifications are valid
         self.add_comment_to_issue(self.issue_number, 'The specifications of your simulator is valid!')
 
         # validate image
@@ -191,6 +208,76 @@ class ValidateCommitSimulatorGitHubAction(GitHubAction):
         # return specifications
         return specifications, test_results
 
+    def validate_permissions(self, simulator_id, simulator_name, submitter):
+        """ Validate that the submitter has permissions to submit or update the simulator
+
+        Args:
+            simulator_id (:obj:`str`): simulator id
+            simulator_name (:obj:`str`): simulator name
+            submitter (:obj:`str`): GitHub id of the submitter
+        """
+        # check if team exists
+        response = requests.get(
+            'https://api.github.com/orgs/biosimulators/teams/' + simulator_id,
+            auth=self.get_gh_auth())
+        try:
+            response.raise_for_status()
+            has_team = True
+        except requests.exceptions.HTTPError:
+            if response.status_code == 404:
+                has_team = False
+            else:
+                raise
+
+        # create team if none exists and add submitter as a maintainer of the team
+        if not has_team:
+            # get parent team
+            response = requests.get(
+                'https://api.github.com/orgs/biosimulators/teams/simulator-developers',
+                auth=self.get_gh_auth())
+            response.raise_for_status()
+            base_team_id = response.json()['id']
+
+            # create team
+            response = requests.post(
+                'https://api.github.com/orgs/biosimulators/teams',
+                auth=self.get_gh_auth(), json={
+                    'name': simulator_id,
+                    'description': 'Developers of ' + simulator_name,
+                    'parent_team_id': base_team_id,
+                    'maintainers': [submitter],
+                })
+            response.raise_for_status()
+
+            # tell user a group was created
+            msg = (
+                'We created the GitHub group @biosimulators/{} to manage permissions to change the specifications of {} '
+                'and added you (@{}) to this group. You can manage permissions to change the specifications of {} at '
+                'https://github.com/orgs/biosimulators/teams/{}/members.'
+            ).format(simulator_id, simulator_name, submitter, simulator_name, simulator_id)
+            self.add_comment_to_issue(self.issue_number, msg)
+
+        # check submitter has permissions to team
+        else:
+            response = requests.get(
+                'https://api.github.com/orgs/biosimulators/teams/{}/memberships/{}'.format(simulator_id, submitter),
+                auth=self.get_gh_auth())
+            try:
+                response.raise_for_status()
+                has_permissions = True
+            except requests.exceptions.HTTPError:
+                if response.status_code == 404:
+                    has_permissions = False
+                else:
+                    raise
+
+            if not has_permissions:
+                msg = (
+                    'You (@{}) do not have permissions to update the specifications of {}. Only the members of @biosimulators/{} '
+                    'can update the specifications of {}. Please contact the members of this group to request permissions to update {}.'
+                ).format(submitter, simulator_name, simulator_id, simulator_name, simulator_name)
+                self.add_error_comment_to_issue(self.issue_number, [Comment(text=msg, error=True)])
+
     def validate_image(self, specifications):
         """ Validate a Docker image for simulation tool
 
@@ -215,17 +302,66 @@ class ValidateCommitSimulatorGitHubAction(GitHubAction):
         case_results = validator.run()
         write_test_results(case_results, '.biosimulators-test-suite-results.json',
                            gh_issue=int(self.issue_number), gh_action_run=int(self.get_gh_action_run_id()))
-        summary, failure_details, warning_details = validator.summarize_results(case_results)
-        msg = '## Summary of tests\n\n{}\n\n'.format(summary)
+        summary, failure_details, warning_details, skipped_details = validator.summarize_results(case_results)
+
+        # print summary to console
+        print('')
+        print('=============== SUMMARY ===============')
+        print('')
+        print(summary + '\n\n')
         if failure_details:
-            msg += '\n## Failures\n\n{}\n\n'.format('### ' + '\n### '.join(failure_details))
+            color = Colors.failure.value
+            print(termcolor.colored('=============== FAILURES ===============', color))
+            print(termcolor.colored('', color))
+            print(termcolor.colored('* ' + '\n\n* '.join(failure_details), color))
+            print('')
         if warning_details:
-            msg += '\n## Warnings\n\n{}\n\n'.format('### ' + '\n### '.join(warning_details))
-        self.add_comment_to_issue(self.issue_number, msg)
+            color = Colors.warning.value
+            print(termcolor.colored('=============== WARNINGS ===============', color))
+            print(termcolor.colored('', color))
+            print(termcolor.colored('* ' + '\n\n* '.join(warning_details), color))
+            print('')
+        if skipped_details:
+            color = Colors.skipped.value
+            print(termcolor.colored('================ SKIPS =================', color))
+            print(termcolor.colored('', color))
+            print(termcolor.colored('* ' + '\n\n* '.join(skipped_details), color))
+            print('')
+
+        # push summary to comments on GitHub issue
+        unable_to_post_results_msg = (
+            'The summary of the tests of your Docker image could not be posted to this GitHub issue. '
+            'The most likely reason is that the summary is too long to post to a comment on a GitHub issue. '
+            'Please use the [console log]({}) of the associated GitHub Action to see the summary of the tests of your '
+            'Docker image.'
+        ).format(self.get_gh_action_run_url())
+
+        msg = '## Summary of tests\n\n{}\n\n'.format(summary)
+        self.add_comment_to_issue(self.issue_number, msg, alternative_comment=unable_to_post_results_msg)
+
+        if failure_details:
+            msg = '\n## Failures\n\n{}\n\n'.format('### ' + '\n### '.join(failure_details))
+            self.add_comment_to_issue(self.issue_number, msg, alternative_comment=unable_to_post_results_msg)
+
+        if warning_details:
+            msg = '\n## Warnings\n\n{}\n\n'.format('### ' + '\n### '.join(warning_details))
+            self.add_comment_to_issue(self.issue_number, msg, alternative_comment=unable_to_post_results_msg)
+
+        if skipped_details:
+            msg = '\n## Skips\n\n{}\n\n'.format('### ' + '\n### '.join(skipped_details))
+            self.add_comment_to_issue(self.issue_number, msg, alternative_comment=unable_to_post_results_msg)
 
         invalid_cases = [case_result for case_result in case_results if case_result.type == TestCaseResultType.failed]
         if invalid_cases:
-            error_msg = 'After correcting your simulator, please edit the first block of this issue to re-initiate this validation.'
+            gh_action_run_url = self.get_gh_action_run_url()
+            error_msg = (
+                'After correcting your simulator, please edit the first block of this issue to re-initiate this validation.\n\n'
+                'The complete log of your validation/submission job, including further information about the failure, '
+                'is available [here]({}). The results of the validation of your image will also be '
+                'available shortly as a JSON file. A link to this file will be available from the "Artifacts" '
+                'section at the bottom of [this page]({}).'
+            ).format(gh_action_run_url, gh_action_run_url)
+
             self.add_error_comment_to_issue(self.issue_number, [Comment(text=error_msg, error=True)])
 
         valid_cases = [case_result for case_result in case_results
@@ -429,3 +565,24 @@ class ValidateCommitSimulatorGitHubAction(GitHubAction):
         response.raise_for_status()
         response_data = response.json()
         return {'Authorization': response_data['token_type'] + ' ' + response_data['access_token']}
+
+    @classmethod
+    def add_comment_to_issue(cls, issue_number, comment, alternative_comment=None, max_len=65536):
+        """ Post a comment to the GitHub issue
+
+        Args:
+            issue_number (:obj:`str`): issue number
+            comment (:obj:`str`): comment
+            alternative_comment (:obj:`str`, optional): optional alternative comment to try posting to the GitHub issue
+            max_len (:obj:`int`, optional): maximum comment length accepted by GitHub
+        """
+        try:
+            if len(comment) > max_len:
+                comment = comment[0:max_len - 4] + ' ...'
+            super(ValidateCommitSimulatorGitHubAction, cls).add_comment_to_issue(issue_number, comment)
+
+        except (requests.exceptions.RequestException, requests.exceptions.ProxyError, requests.exceptions.SSLError):
+            if alternative_comment:
+                super(ValidateCommitSimulatorGitHubAction, cls).add_comment_to_issue(issue_number, alternative_comment)
+            else:
+                raise
