@@ -9,7 +9,7 @@
 from ..config import Config
 from ..data_model import (TestCase, SedTaskRequirements, ExpectedSedReport, ExpectedSedDataSet, ExpectedSedPlot,
                           AlertType, OutputMedium)
-from ..exceptions import InvalidOutputsException, SkippedTestCaseException
+from ..exceptions import InvalidOutputsException, SkippedTestCaseException, TimeoutException
 from ..warnings import IgnoredTestCaseWarning, SimulatorRuntimeErrorWarning, InvalidOutputsWarning
 from .utils import are_array_shapes_equivalent
 from biosimulators_utils.combine.data_model import CombineArchive, CombineArchiveContentFormatPattern  # noqa: F401
@@ -38,6 +38,7 @@ import numpy.testing
 import os
 import re
 import shutil
+import subprocess
 import tempfile
 import types  # noqa: F401
 import warnings
@@ -263,13 +264,16 @@ class SimulatorCanExecutePublishedProject(TestCase):
 
         return True
 
-    def eval(self, specifications, synthetic_archives_dir=None):
+    def eval(self, specifications, synthetic_archives_dir=None, dry_run=False, cli=None):
         """ Evaluate a simulator's performance on a test case
 
         Args:
             specifications (:obj:`dict`): specifications of the simulator to validate
             synthetic_archives_dir (:obj:`str`, optional): Directory to save the synthetic COMBINE/OMEX archives
                 generated by the test cases
+            dry_run (:obj:`bool`): if :obj:`True`, do not use the simulator to execute COMBINE/OMEX archives.
+            cli (:obj:`str`, optional): command-line interface to use to execute the tests involving the simulation of COMBINE/OMEX
+                archives rather than a Docker image
 
         Raises:
             :obj:`SkippedTestCaseException`: if the test case is not applicable to the simulator
@@ -284,16 +288,35 @@ class SimulatorCanExecutePublishedProject(TestCase):
             raise SkippedTestCaseException('Case requires model formats {} and simulation algorithms {}'.format(
                 ', '.join(sorted(model_formats)), ', '.join(sorted(simulation_algorithms))))
 
+        if dry_run:
+            return
+
         # create output directory for case
         out_dir = tempfile.mkdtemp()
 
         # pull image and execute COMBINE/OMEX archive for case
-        pull_docker_image = Config().pull_docker_image
+        config = Config()
+        pull_docker_image = config.pull_docker_image
+        user_to_exec_within_container = config.user_to_exec_in_simulator_containers
         try:
-            biosimulators_utils.simulator.exec.exec_sedml_docs_in_archive_with_containerized_simulator(
-                self.filename, out_dir, specifications['image']['url'], pull_docker_image=pull_docker_image)
+            if os.getenv('CI', 'false').lower() in ['1', 'true']:
+                user_to_exec_within_container = '_SUDO_'
+
+            if cli:
+                biosimulators_utils.simulator.exec.exec_sedml_docs_in_archive_with_simulator_cli(
+                    self.filename, out_dir, cli)
+            else:
+                biosimulators_utils.simulator.exec.exec_sedml_docs_in_archive_with_containerized_simulator(
+                    self.filename, out_dir, specifications['image']['url'], pull_docker_image=pull_docker_image,
+                    user_to_exec_within_container=user_to_exec_within_container)
+
+                if os.path.isdir(out_dir) and os.getenv('CI', 'false').lower() in ['1', 'true']:
+                    subprocess.run(['sudo', 'chown', '{}:{}'.format(os.getuid(), os.getgid()), '-R', out_dir], check=True)
 
         except Exception as exception:
+            if os.path.isdir(out_dir) and os.getenv('CI', 'false').lower() in ['1', 'true']:
+                subprocess.run(['sudo', 'chown', '{}:{}'.format(os.getuid(), os.getgid()), '-R', out_dir], check=True)
+
             shutil.rmtree(out_dir)
             if self.runtime_failure_alert_type == AlertType.exception:
                 raise
@@ -441,13 +464,16 @@ class SyntheticCombineArchiveTestCase(TestCase):
         self.published_projects_test_cases = published_projects_test_cases or []
         self._published_projects_test_case = None
 
-    def eval(self, specifications, synthetic_archives_dir=None):
+    def eval(self, specifications, synthetic_archives_dir=None, dry_run=False, cli=None):
         """ Evaluate a simulator's performance on a test case
 
         Args:
             specifications (:obj:`dict`): specifications of the simulator to validate
             synthetic_archives_dir (:obj:`str`, optional): Directory to save the synthetic COMBINE/OMEX archives
                 generated by the test cases
+            dry_run (:obj:`bool`): if :obj:`True`, do not use the simulator to execute COMBINE/OMEX archives.
+            cli (:obj:`str`, optional): command-line interface to use to execute the tests involving the simulation of COMBINE/OMEX
+                archives rather than a Docker image
 
         Returns:
             :obj:`bool`: whether there were no warnings about the outputs
@@ -456,22 +482,25 @@ class SyntheticCombineArchiveTestCase(TestCase):
             :obj:`Exception`: if the simulator did not pass the test case
         """
         try:
-            return_value = self._eval(specifications, synthetic_archives_dir=synthetic_archives_dir)
+            return_value = self._eval(specifications, synthetic_archives_dir=synthetic_archives_dir, dry_run=dry_run, cli=cli)
         except Exception as exception:
-            if self.REPORT_ERROR_AS_SKIP:
+            if not isinstance(exception, TimeoutException) and self.REPORT_ERROR_AS_SKIP:
                 raise SkippedTestCaseException(str(exception))
             else:
                 raise
 
         return return_value
 
-    def _eval(self, specifications, synthetic_archives_dir=None):
+    def _eval(self, specifications, synthetic_archives_dir=None, dry_run=False, cli=None):
         """ Evaluate a simulator's performance on a test case
 
         Args:
             specifications (:obj:`dict`): specifications of the simulator to validate
             synthetic_archives_dir (:obj:`str`, optional): Directory to save the synthetic COMBINE/OMEX archives
                 generated by the test cases
+            dry_run (:obj:`bool`): if :obj:`True`, do not use the simulator to execute COMBINE/OMEX archives.
+            cli (:obj:`str`, optional): command-line interface to use to execute the tests involving the simulation of COMBINE/OMEX
+                archives rather than a Docker image
 
         Returns:
             :obj:`bool`: whether there were no warnings about the outputs
@@ -525,17 +554,19 @@ class SyntheticCombineArchiveTestCase(TestCase):
         try:
             for i_archive, expected_results_of_synthetic_archive in enumerate(expected_results_of_synthetic_archives):
                 if self._eval_synthetic_archive(specifications, expected_results_of_synthetic_archive, shared_archive_dir,
-                                                i_archive, synthetic_archives_dir=synthetic_archives_dir):
+                                                i_archive, synthetic_archives_dir=synthetic_archives_dir, dry_run=dry_run,
+                                                cli=cli):
                     has_warnings = True
         finally:
             shutil.rmtree(temp_dir)
         return not has_warnings
 
     def _eval_synthetic_archive(self, specifications, expected_results_of_synthetic_archive, shared_archive_dir,
-                                i_synthetic_archive, synthetic_archives_dir=None):
+                                i_synthetic_archive, synthetic_archives_dir=None, dry_run=False, cli=None):
         synthetic_archive = expected_results_of_synthetic_archive.archive
         synthetic_sed_docs = expected_results_of_synthetic_archive.sed_documents
         is_success_expected = expected_results_of_synthetic_archive.is_success_expected
+        environment = expected_results_of_synthetic_archive.environment
 
         sedml_writer = SedmlSimulationWriter()
         temp_dir = tempfile.mkdtemp()
@@ -558,13 +589,31 @@ class SyntheticCombineArchiveTestCase(TestCase):
                                                                  'execute-should-fail'))
             shutil.copy(synthetic_archive_filename, export_synthetic_archive_filename)
 
+        if dry_run:
+            shutil.rmtree(temp_dir)
+            return False
+
         # use synthetic archive to test simulator
         outputs_dir = os.path.join(temp_dir, 'outputs')
-        pull_docker_image = Config().pull_docker_image
+        config = Config()
+        pull_docker_image = config.pull_docker_image
+        user_to_exec_within_container = config.user_to_exec_in_simulator_containers
         has_warnings = False
         try:
-            biosimulators_utils.simulator.exec.exec_sedml_docs_in_archive_with_containerized_simulator(
-                synthetic_archive_filename, outputs_dir, specifications['image']['url'], pull_docker_image=pull_docker_image)
+            if os.getenv('CI', 'false').lower() in ['1', 'true']:
+                user_to_exec_within_container = '_SUDO_'
+
+            if cli:
+                biosimulators_utils.simulator.exec.exec_sedml_docs_in_archive_with_simulator_cli(
+                    synthetic_archive_filename, outputs_dir, cli, environment=environment)
+            else:
+                biosimulators_utils.simulator.exec.exec_sedml_docs_in_archive_with_containerized_simulator(
+                    synthetic_archive_filename, outputs_dir, specifications['image']['url'], pull_docker_image=pull_docker_image,
+                    environment=environment,
+                    user_to_exec_within_container=user_to_exec_within_container)
+
+            if os.path.isdir(outputs_dir) and os.getenv('CI', 'false').lower() in ['1', 'true']:
+                subprocess.run(['sudo', 'chown', '{}:{}'.format(os.getuid(), os.getgid()), '-R', outputs_dir], check=True)
 
             if not self.eval_outputs(specifications, synthetic_archive, synthetic_sed_docs, outputs_dir):
                 has_warnings = True
@@ -572,6 +621,9 @@ class SyntheticCombineArchiveTestCase(TestCase):
             succeeded = True
 
         except Exception:
+            if os.path.isdir(outputs_dir) and os.getenv('CI', 'false').lower() in ['1', 'true']:
+                subprocess.run(['sudo', 'chown', '{}:{}'.format(os.getuid(), os.getgid()), '-R', outputs_dir], check=True)
+
             succeeded = False
             if is_success_expected:
                 shutil.rmtree(temp_dir)
@@ -703,7 +755,7 @@ class SyntheticCombineArchiveTestCase(TestCase):
             if (
                 simulation.initial_time != 0
                 or simulation.output_start_time != 0
-                or simulation.number_of_points <= 0
+                or simulation.number_of_points < 1
                 or int(simulation.number_of_points / 2) != simulation.number_of_points / 2
             ):
                 return False
@@ -777,12 +829,14 @@ class ExpectedResultOfSyntheticArchive(object):
         sed_documents (:obj:`dict` of :obj:`str` to :obj:`SedDocument`): map from locations to
           SED documents in synthetic archive
         is_success_expected (:obj:`bool`, optional): whether the execution of the archive is expected to succeed
+        environment (:obj:`dict`, optional): environment variables that the archive should be executed with
     """
 
-    def __init__(self, archive, sed_documents, is_success_expected=True):
+    def __init__(self, archive, sed_documents, is_success_expected=True, environment=None):
         self.archive = archive
         self.sed_documents = sed_documents
         self.is_success_expected = is_success_expected
+        self.environment = environment or {}
 
 
 def find_cases(specifications, dir_name=None, output_medium=OutputMedium.console):
@@ -813,6 +867,9 @@ def find_cases(specifications, dir_name=None, output_medium=OutputMedium.console
         if case.compatible_with_specifications(specifications):
             compatible_cases.append(case)
 
+    all_cases.sort(key=lambda case: case.filename)
+    compatible_cases.sort(key=lambda case: case.filename)
+
     # return cases
     return (all_cases, compatible_cases)
 
@@ -914,6 +971,14 @@ class ConfigurableMasterCombineArchiveTestCase(SyntheticCombineArchiveTestCase):
                     if key_report:
                         break
 
+            key_sim = key_task.simulation
+            if isinstance(key_sim, UniformTimeCourseSimulation):
+                key_sim.output_end_time = (
+                    key_sim.output_start_time
+                    + 10 / key_sim.number_of_points * (key_sim.output_end_time - key_sim.output_start_time)
+                )
+                key_sim.number_of_points = 10
+
             doc.models = [key_task.model]
             doc.simulations = [key_task.simulation]
             doc.tasks = [key_task]
