@@ -21,6 +21,9 @@ from biosimulators_utils.sedml.data_model import (SedDocument, Task, Output, Rep
                                                   RepeatedTask, Range, UniformRange, UniformRangeType, VectorRange, FunctionalRange,
                                                   SetValueComputeModelChange, SubTask)
 from biosimulators_utils.sedml.utils import get_xml_node_namespace_tag_target
+from kisao import Kisao
+from kisao.data_model import AlgorithmSubstitutionPolicy
+from kisao.utils import get_substitutable_algorithms_for_policy
 from lxml import etree
 import abc
 import copy
@@ -28,6 +31,7 @@ import numpy
 import numpy.testing
 import os
 import PyPDF2
+import re
 import shutil
 import tempfile
 import warnings
@@ -55,6 +59,7 @@ __all__ = [
     'SimulatorProducesLinear3DPlots',
     'SimulatorProducesLogarithmic3DPlots',
     'SimulatorProducesMultiplePlots',
+    'SimulatorSupportsSubstitutingAlgorithms',
 ]
 
 
@@ -275,6 +280,11 @@ class SimulatorSupportsModelAttributeChanges(SimulatorSupportsModelsSimulationsT
             )
 
             for key, value in node.attrib.items():
+                if key[0] == '{':
+                    ns, _, key = key[1:].rpartition('}')
+                    rev_namespaces = {v: k for k, v in node.nsmap.items()}
+                    key = rev_namespaces[ns] + ':' + key
+
                 model_1.changes.append(
                     ModelAttributeChange(
                         target=node_target + '/@' + key,
@@ -379,6 +389,11 @@ class SimulatorSupportsComputeModelChanges(SimulatorSupportsModelsSimulationsTas
             )
 
             for key, value in node.attrib.items():
+                if key[0] == '{':
+                    ns, _, key = key[1:].rpartition('}')
+                    rev_namespaces = {v: k for k, v in node.nsmap.items()}
+                    key = rev_namespaces[ns] + ':' + key
+
                 try:
                     float(value)
                 except ValueError:
@@ -392,7 +407,7 @@ class SimulatorSupportsComputeModelChanges(SimulatorSupportsModelsSimulationsTas
                         target=node_target + '/@' + key,
                         target_namespaces=target_namespaces,
                         parameters=[
-                            Parameter(id=param_id, value=1e100)
+                            Parameter(id=param_id, value=-1)
                         ],
                         variables=[
                             Variable(
@@ -411,7 +426,7 @@ class SimulatorSupportsComputeModelChanges(SimulatorSupportsModelsSimulationsTas
                         target=node_target + '/@' + key,
                         target_namespaces=target_namespaces,
                         parameters=[
-                            Parameter(id=param_id, value=1e100)
+                            Parameter(id=param_id, value=-1)
                         ],
                         variables=[
                             Variable(
@@ -722,7 +737,7 @@ class SimulatorSupportsMultipleTasksPerSedDocument(SingleMasterSedDocumentCombin
 
                         for var in data_set.data_generator.variables:
                             copy_var = Variable(
-                                id=var.id,
+                                id=var.id + '__test_suite_copy',
                                 target=var.target,
                                 target_namespaces=var.target_namespaces,
                                 symbol=var.symbol,
@@ -730,6 +745,8 @@ class SimulatorSupportsMultipleTasksPerSedDocument(SingleMasterSedDocumentCombin
                             )
                             copy_var.task = copy_tasks[var.task.id]
                             copy_data_gen.variables.append(copy_var)
+
+                            copy_data_gen.math = re.sub(r'((^|\b){}(\b|$))'.format(var.id), copy_var.id, copy_data_gen.math)
                     copy_data_set.data_generator = copy_data_gen
 
         # return modified SED document
@@ -801,7 +818,11 @@ class SimulatorSupportsMultipleReportsPerSedDocument(SingleMasterSedDocumentComb
             sed_doc.outputs[i_dataset % 2].data_sets.append(data_set)
 
         # to ensure that each report has at least one data set, including when there's only 1 data set total
-        sed_doc.outputs[1].data_sets.append(original_data_sets[0])
+        sed_doc.outputs[1].data_sets.append(DataSet(
+            id=original_data_sets[0].id + '__copy__',
+            label=original_data_sets[0].label,
+            data_generator=original_data_sets[0].data_generator,
+        ))
 
         # return modified SED document
         return expected_results_of_synthetic_archives
@@ -928,6 +949,11 @@ class RepeatedTasksTestCase(SimulatorSupportsModelsSimulationsTasksDataGenerator
                 )
 
                 for key, value in node.attrib.items():
+                    if key[0] == '{':
+                        ns, _, key = key[1:].rpartition('}')
+                        rev_namespaces = {v: k for k, v in node.nsmap.items()}
+                        key = rev_namespaces[ns] + ':' + key
+
                     try:
                         float(value)
                     except ValueError:
@@ -992,7 +1018,7 @@ class RepeatedTasksTestCase(SimulatorSupportsModelsSimulationsTasksDataGenerator
                     ranges[-1].parameters.append(
                         Parameter(
                             id='__repeated_task_range_p_' + str(i_repeated_task) + '_' + str(i_range + 1) + '_2',
-                            value=1e100 if breaking_range else 0.,
+                            value=-1 if breaking_range else 0.,
                         ),
                     )
                     ranges[-1].variables.append(
@@ -1048,6 +1074,11 @@ class RepeatedTasksTestCase(SimulatorSupportsModelsSimulationsTasksDataGenerator
             )
 
             for key, value in node.attrib.items():
+                if key[0] == '{':
+                    ns, _, key = key[1:].rpartition('}')
+                    rev_namespaces = {v: k for k, v in node.nsmap.items()}
+                    key = rev_namespaces[ns] + ':' + key
+
                 try:
                     original_value = float(value)
                 except ValueError:
@@ -1066,7 +1097,7 @@ class RepeatedTasksTestCase(SimulatorSupportsModelsSimulationsTasksDataGenerator
                             ),
                             Parameter(
                                 id='p_1_' + str(len(changes)) + '_' + str(i_repeated_task),
-                                value=1. if self.FUNCTONAL_RANGE_USES_VARIABLES else 1e100,
+                                value=1. if self.FUNCTONAL_RANGE_USES_VARIABLES else -1,
                             ),
                         ],
                         variables=[
@@ -1206,7 +1237,12 @@ class RepeatedTasksTestCase(SimulatorSupportsModelsSimulationsTasksDataGenerator
                 sed_doc.data_generators.append(data_gen_2)
 
                 variable = data_gen.variables[0]
-                variable_2 = Variable(id=data_gen_2.math, symbol=variable.symbol, target=variable.target, task=repeated_task)
+                variable_2 = Variable(
+                    id=data_gen_2.math,
+                    symbol=variable.symbol,
+                    target=variable.target,
+                    target_namespaces=variable.target_namespaces,
+                    task=repeated_task)
                 data_gen_2.variables.append(variable_2)
 
         # return modified SED document
@@ -1236,6 +1272,10 @@ class RepeatedTasksTestCase(SimulatorSupportsModelsSimulationsTasksDataGenerator
         for data_set, repeated_data_set in zip(report.data_sets, repeated_report.data_sets):
             results_data_set = results[data_set.id]
             repeated_results_data_set = repeated_results[repeated_data_set.id]
+
+            if results_data_set.ndim == 0:
+                results_data_set = results_data_set.reshape((1,))
+
             if repeated_results_data_set.ndim - results_data_set.ndim != 2 * (self.NUM_NESTED_REPEATED_TASKS + 1):
                 raise InvalidOutputsException('Each level of repeated task should contribute two additional dimensions to reports')
 
@@ -1330,8 +1370,7 @@ class SimulatorSupportsRepeatedTasksWithLinearUniformRanges(RepeatedTasksTestCas
     NUM_SUB_TASKS = 1
     NUM_NESTED_REPEATED_TASKS = 0
 
-    # TODO: Uncomment to enable test case when libSED-ML fixed; libSED-ML 2.0.14 cannot output uniform ranges with L1V3
-    # def is_concrete(self): return True
+    def is_concrete(self): return True
 
 
 class SimulatorSupportsRepeatedTasksWithLogarithmicUniformRanges(RepeatedTasksTestCase):
@@ -1341,8 +1380,7 @@ class SimulatorSupportsRepeatedTasksWithLogarithmicUniformRanges(RepeatedTasksTe
     NUM_SUB_TASKS = 1
     NUM_NESTED_REPEATED_TASKS = 0
 
-    # TODO: Uncomment to enable test case when libSED-ML fixed; libSED-ML 2.0.14 cannot output uniform ranges with L1V3
-    # def is_concrete(self): return True
+    def is_concrete(self): return True
 
 
 class SimulatorSupportsRepeatedTasksWithVectorRanges(RepeatedTasksTestCase):
@@ -1425,7 +1463,7 @@ class SimulatorSupportsRepeatedTasksWithSubTasksOfMixedTypes(RepeatedTasksTestCa
     different shapes.
     """
     RANGE_TYPE = VectorRange
-    RANGE_LENS = (1, 3)
+    RANGE_LENS = (2, 3)
     NUM_SUB_TASKS = 2
     NUM_NESTED_REPEATED_TASKS = 1
     MIXED_SUB_TASK_TYPES = True
@@ -1533,6 +1571,27 @@ class SimulatorProducesPlotsTestCase(SingleMasterSedDocumentCombineArchiveTestCa
                 '\n  - '.join(sorted('`' + id + '`' for id in extra_plot_ids)))
             warnings.warn(msg, InvalidOutputsWarning)
 
+        # check plot data saved
+        expected_plot_ids = set()
+        for doc_location, doc in synthetic_sed_docs.items():
+            doc_id = os.path.relpath(doc_location, './')
+            for output in doc.outputs:
+                if isinstance(output, (Plot2D, Plot3D)):
+                    expected_plot_ids.add(os.path.join(doc_id, output.id))
+
+        try:
+            plot_ids = ReportReader().get_ids(outputs_dir)
+        except Exception:
+            plot_ids = []
+
+        missing_plot_ids = expected_plot_ids.difference(set(plot_ids))
+
+        if missing_plot_ids:
+            raise InvalidOutputsException('Simulator did not produce data for the following plots:\n  - {}'.format(
+                '\n  - '.join(sorted('`' + id + '`' for id in missing_plot_ids))
+            ))
+
+        # remove temporary directory
         shutil.rmtree(tempdir)
 
 
@@ -1724,7 +1783,7 @@ class SimulatorSupportsDataGeneratorsWithDifferentShapes(UniformTimeCourseTestCa
             id=sim.id + '__copy_2',
             initial_time=sim.initial_time,
             output_start_time=sim.output_start_time,
-            output_end_time=sim.output_end_time,
+            output_end_time=sim.output_end_time + (sim.output_end_time - sim.output_start_time),
             number_of_points=sim.number_of_points * 2,
             algorithm=copy.deepcopy(sim.algorithm),
         )
@@ -1844,7 +1903,7 @@ class SimulatorSupportsDataSetsWithDifferentShapes(UniformTimeCourseTestCase):
             id=sim.id + '__copy_2',
             initial_time=sim.initial_time,
             output_start_time=sim.output_start_time,
-            output_end_time=sim.output_end_time,
+            output_end_time=sim.output_end_time + (sim.output_end_time - sim.output_start_time),
             number_of_points=sim.number_of_points * 2,
             algorithm=copy.deepcopy(sim.algorithm),
         )
@@ -1919,11 +1978,7 @@ class SimulatorSupportsDataSetsWithDifferentShapes(UniformTimeCourseTestCase):
                 expected_length = sim2.number_of_points + 1
 
                 if data_set.data_generator.variables[0].symbol:
-                    data_set_slice = tuple(
-                        [slice(0, dim_len) for dim_len in value.shape[0:-1]]
-                        + [slice(0, sim2.number_of_points + 1, 2)]
-                    )
-                    values2[data_set.data_generator.id.replace('__copy_2', '')] = value[data_set_slice]
+                    values2[data_set.data_generator.id.replace('__copy_2', '')] = value[0:sim1.number_of_points + 1]
 
             self._eval_data_set(data_set.id, value, expected_length)
 
@@ -1943,3 +1998,96 @@ class SimulatorSupportsDataSetsWithDifferentShapes(UniformTimeCourseTestCase):
         except Exception as exception:
             raise InvalidOutputsException('Simulations with the same time courses should produce equivalent time data sets:\n\n  {}'.format(
                 str(exception).replace('\n', '\n  ')))
+
+
+class SimulatorSupportsSubstitutingAlgorithms(SimulatorSupportsModelsSimulationsTasksDataGeneratorsAndReports):
+    """ Check that a simulator can substitute algorithms that it doesn't implement with similar
+    algorithms when the algorithm substitution policy is less restrictive than
+    :obj:`AlgorithmSubstitutionPolicy.SAME_METHOD`. Also check that a simulator ignores unsupported
+    algorithm parameters when the algorithm substitution policy is less restrictive than
+    :obj:`AlgorithmSubstitutionPolicy.NONE`.
+    """
+    REPORT_ERROR_AS_SKIP = True
+
+    def is_curated_sed_algorithm_suitable_for_building_synthetic_archive(self, specifications, algorithm):
+        """ Determine if a SED algorithm is suitable for testing
+
+        Args:
+            specifications (:obj:`dict`): specifications of the simulator to validate
+            algorithm (:obj:`Algorithm`): SED algorithm in curated archive
+
+        Returns:
+            :obj:`bool`: whether the algorithm is suitable for testing
+        """
+        kisao = Kisao()
+        alg_term = kisao.get_term(algorithm.kisao_id)
+        sub_algs = get_substitutable_algorithms_for_policy(
+            alg_term,
+            substitution_policy=AlgorithmSubstitutionPolicy.SIMILAR_VARIABLES)
+        sub_alg_ids = kisao.get_term_ids(sub_algs)
+
+        for alg_specs in specifications['algorithms']:
+            if alg_specs['kisaoId']['id'] in sub_alg_ids:
+                sub_alg_ids.remove(alg_specs['kisaoId']['id'])
+
+        if sub_alg_ids:
+            if 'KISAO_0000019' in sub_alg_ids:
+                self._alt_alg = 'KISAO_0000019'
+            elif 'KISAO_0000088' in sub_alg_ids:
+                self._alt_alg = 'KISAO_0000088'
+            else:
+                self._alt_alg = sub_alg_ids[0]
+            return True
+        else:
+            return False
+
+    def build_synthetic_archives(self, specifications, curated_archive, curated_archive_dir, curated_sed_docs):
+        """ Generate a synthetic archive with master and non-master SED documents
+
+        Args:
+            specifications (:obj:`dict`): specifications of the simulator to validate
+            curated_archive (:obj:`CombineArchive`): curated COMBINE/OMEX archive
+            curated_archive_dir (:obj:`str`): directory with the contents of the curated COMBINE/OMEX archive
+            curated_sed_docs (:obj:`dict` of :obj:`str` to :obj:`SedDocument`): map from locations to
+                SED documents in curated archive
+
+        Returns:
+            :obj:`list` of :obj:`ExpectedResultOfSyntheticArchive`
+        """
+        super(SimulatorSupportsSubstitutingAlgorithms, self).build_synthetic_archives(
+            specifications, curated_archive, curated_archive_dir, curated_sed_docs)
+
+        sed_docs_1 = copy.deepcopy(curated_sed_docs)
+        doc = list(sed_docs_1.values())[0]
+        doc.simulations[0].algorithm.kisao_id = self._alt_alg
+        doc.simulations[0].algorithm.changes = []
+
+        sed_docs_2 = copy.deepcopy(curated_sed_docs)
+        doc = list(sed_docs_2.values())[0]
+        doc.simulations[0].algorithm.changes.append(AlgorithmParameterChange(
+            kisao_id='KISAO_0000000',
+            new_value='0.1',
+        ))
+
+        return [
+            ExpectedResultOfSyntheticArchive(
+                curated_archive,
+                sed_docs_1,
+                False,
+                environment={'ALGORITHM_SUBSTITUTION_POLICY': AlgorithmSubstitutionPolicy.NONE.name}),
+            ExpectedResultOfSyntheticArchive(
+                curated_archive,
+                sed_docs_1,
+                True,
+                environment={'ALGORITHM_SUBSTITUTION_POLICY': AlgorithmSubstitutionPolicy.SIMILAR_VARIABLES.name}),
+            ExpectedResultOfSyntheticArchive(
+                curated_archive,
+                sed_docs_2,
+                False,
+                environment={'ALGORITHM_SUBSTITUTION_POLICY': AlgorithmSubstitutionPolicy.NONE.name}),
+            ExpectedResultOfSyntheticArchive(
+                curated_archive,
+                sed_docs_2,
+                True,
+                environment={'ALGORITHM_SUBSTITUTION_POLICY': AlgorithmSubstitutionPolicy.SIMILAR_VARIABLES.name}),
+        ]
